# -*- coding: utf-8 -*-
# 15/9/19
# create by: snower

import greenlet
from tornado.gen import coroutine as tornado_coroutine, _make_coroutine_wrapper
from tornado.gen import *

def engine(func):
    func = _make_coroutine_wrapper(func, replace_callback=False)

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        child_gr = greenlet.getcurrent()
        main = child_gr.parent

        if main is None:
            future = func(*args, **kwargs)
            def final_callback(future):
                if future.result() is not None:
                    raise ReturnValueIgnoredError(
                        "@gen.engine functions cannot return values: %r" %
                        (future.result(),))
            future.add_done_callback(stack_context.wrap(final_callback))
            return

        def run():
            future = func(*args, **kwargs)
            def final_callback(future):
                if future._exc_info is not None:
                    return child_gr.throw(future.exception())
                elif future.result() is not None:
                    return child_gr.throw(ReturnValueIgnoredError("@gen.engine functions cannot return values: %r" %(future.result(),)))
                return child_gr.switch(None)
            future.add_done_callback(stack_context.wrap(final_callback))
        IOLoop.current().add_callback(run)
        return main.switch()
    return wrapper

def coroutine(func, replace_callback=True):
    func = tornado_coroutine(func, replace_callback)

    @functools.wraps(func)
    def _(*args, **kwargs):
        child_gr = greenlet.getcurrent()
        main = child_gr.parent
        if main is None:
            return func(*args, **kwargs)

        ioloop = IOLoop.current()
        def run():
            future = func(*args, **kwargs)
            def finish(future):
                if future._exc_info is not None:
                    return child_gr.throw(future.exception())
                return child_gr.switch(future.result())
            ioloop.add_future(future, finish)
        ioloop.add_callback(run)
        return main.switch()
    return _


# -*- coding: utf-8 -*-
#14-6-11
# create by: snower

import logging
import greenlet
from tornado import tcpserver
from tornado.iostream import StreamClosedError

class TTornadoServer(tcpserver.TCPServer):
    def __init__(self, processor, input_transport_factory, input_protocol_factory, output_transport_factory = None, output_protocol_factory = None):
        super(TTornadoServer,self).__init__()
        self.processor = processor
        self.input_transport_factory = input_transport_factory
        self.output_transport_factory = output_transport_factory
        self.input_protocol_factory = input_protocol_factory
        self.output_protocol_factory = output_protocol_factory

    def process(self, itrans, otrans):
        iprot = self.input_protocol_factory.getProtocol(itrans)
        oprot = self.output_protocol_factory.getProtocol(otrans) if otrans and self.output_protocol_factory else iprot
        while True:
            try:
                self.processor.process(iprot, oprot)
            except (IOError, StreamClosedError):
                itrans.close()
                if otrans:
                    otrans.close()
                break
            except Exception as e:
                logging.exception("processor error: %s", e)
                itrans.close()
                if otrans:
                    otrans.close()
                break

    def handle_stream(self, stream, address):
        itrans = self.input_transport_factory.getTransport(stream)
        otrans = self.output_transport_factory.getTransport(stream) if self.output_transport_factory else None
        child_gr = greenlet.greenlet(self.process)
        child_gr.switch(itrans, otrans)


# -*- coding: utf-8 -*-
# 14-8-16
# create by: snower

import time
import socket
from tornado.iostream import IOStream

class TStreamConnectTimeoutError(Exception): pass

class TStream(IOStream):
    def __init__(self, host='127.0.0.1', port=9090, unix_socket=None, socket_family=socket.AF_UNSPEC, timeout=5):
        self._host = host
        self._port = port
        self._unix_socket = unix_socket
        self._socket_family = socket_family
        self._timeout = timeout

        if self._unix_socket:
            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        else:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)

        super(TStream, self).__init__(sock)

    def open(self):
        if self._unix_socket:
            address = self._unix_socket
        else:
            address = (self._host, self._port)
        future = self.connect(address)
        if self._timeout > 0:
            def timeout():
                if self._connecting:
                    self.close((None, TStreamConnectTimeoutError(), None))
            self.io_loop.add_timeout(time.time() + self._timeout, timeout)
        return future


# -*- coding: utf-8 -*-
#14-6-11
# create by: snower

import sys
import greenlet
from tornado.concurrent import TracebackFuture
from tornado.ioloop import IOLoop
from thrift.transport import TTransport

if sys.version_info[0] >= 3:
    import io
    StringIO = io.BytesIO
else:
    import cStringIO
    StringIO = cStringIO.StringIO


class TIOStreamTransportFactory:
    """Factory transport that builds buffered transports"""

    def getTransport(self, stream):
        return TIOStreamTransport(stream)


class TIOStreamTransport(TTransport.TTransportBase):
    DEFAULT_BUFFER = 4096

    def __init__(self, stream):
        self._stream = stream
        self._wbuffer = StringIO()
        self._rbuffer = StringIO(b'')
        self._wbuffer_size = 0
        self._rbuffer_size = 0
        self._loop = IOLoop.current()

    def open(self):
        try:
            future = self._stream.open()
        except:
            exc_info = sys.exc_info()
            future = TracebackFuture()
            future.set_exc_info(exc_info)
        return future

    def closed(self):
        return self._stream.closed()

    def close(self):
        if not self.closed():
            return self._stream.close()

    def read(self, sz):
        if sz <= self._rbuffer_size:
            self._rbuffer_size -= sz
            return self._rbuffer.read(sz)

        if sz <= self._stream._read_buffer_size + self._rbuffer_size:
            last_buf = b''
            if self._rbuffer_size > 0:
                last_buf += self._rbuffer.read()
            self._rbuffer_size = self._stream._read_buffer_size + self._rbuffer_size - sz
            self._rbuffer = StringIO(last_buf + b''.join(self._stream._read_buffer))
            self._stream._read_buffer.clear()
            self._stream._read_buffer_size = 0
            if self._stream._state and (self._stream._state & self._stream.io_loop.READ == 0):
                self._stream._state |= self._stream.io_loop.READ
                self._stream.io_loop.update_handler(self._stream.fileno(), self._stream._state)
            return self._rbuffer.read(sz)

        child_gr = greenlet.getcurrent()
        main = child_gr.parent
        assert main is not None, "Execut must be running in child greenlet"

        def read_callback(future):
            if future._exc_info is not None:
                return child_gr.throw(future.exception())

            data = future.result()
            last_buf = b''
            if self._rbuffer_size > 0:
                last_buf += self._rbuffer.read()
            self._rbuffer_size = 0
            return child_gr.switch(last_buf + data)
        future = self._stream.read_bytes(sz - self._rbuffer_size)
        self._loop.add_future(future, read_callback)
        return main.switch()

    def write(self, data):
        self._wbuffer.write(data)
        self._wbuffer_size += len(data)
        if self._wbuffer_size >= self.DEFAULT_BUFFER:
            self.flush()

    def flush(self):
        if self._wbuffer_size > 0:
            data = self._wbuffer.getvalue()
            self._wbuffer = StringIO()
            self._wbuffer_size = 0
            self._stream.write(data)